function [Beta, Yhat, Yhatcv, res, rescv, PRESS] = OLS_LOOCV(X, Y)

% Function finding the least squares solution with full leave one out cross
% validation for a data matrix X and corresponding response matrix Y.
%
% -------------------------------------------------------------------------
%
% Inputs:
% X: Data matrix with rows as individual measurements and colums
%    representing each variable. Must be "slim" => # measurements > 
%    # variables.
% Y: Corresponding column vector or matrix of response measurements
%
% -------------------------------------------------------------------------
%
% Outputs:
% Beta:   Least squares regression coefficients, including constant terms.
% Yhat:   Fitted values for the full data set.
% Yhatcv: CV-fitted values for the full data set.
% res:    Residual matrix = Y - Yhat.
% rescv:  CV-residual matrix = Y - Yhatcv.
% PRESS:  Statistic used for comparing prediction ability of model =
%         sum(rescv.^2).
%
% -------------------------------------------------------------------------

% Initializations:
n      = size(X, 1);      % # measurements
m      = size(Y, 2);      % # response variables
X      = [ones(n, 1), X]; % Expanded data matrix to get the constant term.
Yhatcv = zeros(n, m);     % Initialization of Yhatcv

% LS model:
Beta = (X'*X)\(X'*Y); % Regression coefficients
Yhat = X*Beta;        % Fitted response matrix
res  = Y - Yhat;      % Residual matrix

% LOOCV:
for i = 1:n
    inds         = setdiff(1:n, i);                 % Indices where i is removed
    Xred         = X(inds, :);                      % Reduced data matrix
    Bcv          = (Xred'*Xred)\(Xred'*Y(inds, :)); % CV model regression coeff's
    Yhatcv(i, :) = X(i, :)*Bcv;                     % CV-fitted response values
end

rescv = Y - Yhatcv;    % CV residual martix
PRESS = sum(rescv.^2); % PRESS statistic
