function [Beta, Yhat, Yhatcv, res, rescv, PRESS] = OLS_LOOCV_QR(X, Y)

% Function finding the least squares solution with full leave one out cross
% validation for a data matrix X and corresponding response matrix y. Uses
% QR-factorization of the data matrix.
%
% -------------------------------------------------------------------------
%
% Inputs:
% X: Data matrix with rows as individual measurements and colums
%    representing each variable. Must be "slim" => # measurements > 
%    # variables.
% Y: Corresponding column vector or matrix of response measurements.
%
% -------------------------------------------------------------------------
%
% Outputs:
% Beta:   Least squares regression coefficients, including constant terms.
% Yhat:   Fitted values for the full data set.
% Yhatcv: CV-fitted values for the full data set.
% res:    Residual matrix = Y - Yhat.
% rescv:  CV-residual matrix = Y - Yhatcv.
% PRESS:  Statistic used for comparing prediction ability of model =
%         sum(rescv.^2).
%
% -------------------------------------------------------------------------

% Initializations:
n      = size(X, 1);      % # measurements
m      = size(Y, 2);      % # response variables
X      = [ones(n, 1), X]; % Expanded data matrix to get the constant term.
Yhatcv = zeros(n, m);     % Initialization of Yhatcv

% LS model:
[Q, R] = qr(X, 0); % QR factorization where R i square.
Beta   = R\(Q'*Y); % Regression coefficients
Yhat   = Q*(Q'*Y); % Fitted response matrix
res    = Y - Yhat; % Residual matrix

% LOOCV:
for i = 1:n
    inds         = setdiff(1:n, i);         % Indices where i is removed
    [Qred, Rred] = qr(X(inds, :), 0);       % QR-factorization of reduced data
    Bcv          = Rred\(Qred'*Y(inds, :)); % CV model regression coefficients
    Yhatcv(i, :) = X(i, :)*Bcv;             % CV-fitted response values
end

rescv = Y - Yhatcv;    % CV residual matrix
PRESS = sum(rescv.^2); % PRESS statistic