function [Beta, Yhat, Yhatcv, res, rescv, PRESS] = OLS_LOOCV_SVD(X, Y)

% Function finding the least squares solution with full leave one out cross
% validation for a data matrix X and corresponding response matrix Y. Uses
% singular value decomposition of the data matrix.
%
% -------------------------------------------------------------------------
%
% Inputs:
% X: Data matrix with rows as individual measurements and colums
%    representing each variable. Must be "slim" => # measurements > 
%    # variables.
% y: Corresponding column vector or matrix of response measurements.
%
% -------------------------------------------------------------------------
%
% Outputs:
% Beta:   Least squares regression coefficients, including constant terms.
% Yhat:   Fitted values for the full data set.
% Yhatcv: CV-fitted values for the full data set.
% res:    Residual matrix = Y - Yhat
% rescv:  CV-residual matrix = Y - Yhatcv
% PRESS:  Statistic used for comparing prediction ability of model =
%         sum(rescv.^2).
%
% -------------------------------------------------------------------------

% Initializations:
n      = size(X, 1);      % # measurements
m      = size(Y, 2);      % # response variables
X      = [ones(n, 1), X]; % Expanded data matrix to get the constant term.
Yhatcv = zeros(n, m);     % Initialization of yhatcv

% LS model:
[U, S, V] = svd(X, 'econ'); % "Economical" SVD of data matrix
Beta      = V*(S\U')*Y;     % Regression coefficients
Yhat      = U*U'*Y;         % Fitted response matrix
res       = Y - Yhat;       % Residual martix

% LOOCV:
for i = 1:n
    inds         = setdiff(1:n, i);         % Indices where i is removed
    [Ur, Sr, Vr] = svd(X(inds, :), 'econ'); % SVD of reduced data
    bcv          = Vr*(Sr\Ur')*Y(inds, :);  % CV model regression coefficients
    Yhatcv(i, :) = X(i, :)*bcv;             % CV-fitted response value
end

rescv = Y - Yhatcv;    % CV residual matrix
PRESS = sum(rescv.^2); % PRESS statistic