function [Beta, Yhat, Yhatcv, res, h, rescv, PRESS] = OLS_fastLOOCV(X, Y)

% Function finding the least squares solution with fast leave one out cross
% validation for a data matrix X and corresponding response matrix y. 
%
% -------------------------------------------------------------------------
%
% Inputs:
% X: Data matrix with rows as individual measurements and colums
%    representing each variable. Must be "slim" => # measurements > 
%    # variables.
% y: Corresponding column vector or matrix of response measurements.
%
% -------------------------------------------------------------------------
%
% Outputs:
% Beta:   Least squares regression coefficients, including constant terms.
% Yhat:   Fitted values for the full data set.
% Yhatcv: CV-fitted values for the full data set.
% res:    Residual matrix = y - yhat
% h:      Vector of leverage values
% rescv:  CV-residual matrix = y - yhatcv
% PRESS:  Statistic used for comparing prediction ability of model =
%         sum(rescv.^2).
%
% -------------------------------------------------------------------------

% Initializations
n  = size(X, 1);        % # measurements
h  = zeros(n, 1);       % Initialization of h
mX = mean(X);           % Mean data values, one for each variable
mY = mean(Y);           % Mean response value
X  = X - ones(n, 1)*mX; % Centered data matrix
Y  = Y - ones(n, 1)*mY; % Centered response values

Beta = (X'*X)\(X'*Y); % Regression coefficients
Yhat = X*Beta;        % Fitted response vector
res  = Y - Yhat;      % Residual vector

% Leverage
for i = 1:n
    x = X(i, :);                % the i-th sample
    h(i) = x*((X'*X)\x') + 1/n; % corresponding leverage value
end

% LOOCV values
Yhatcv = bsxfun(@rdivide, Yhat - bsxfun(@times, Y, h), 1 - h); % CV fitted values
rescv  = Y - Yhatcv;             % CV residuals
PRESS = sum(rescv.^2);           % PRESS statistic

Beta = [mY - mX*Beta; Beta]; % reg coeffs with constant term
