%% LOOCV Principal component regression.

function [BetaLOOCV, PRESS] = PCR2LOOCV(X, Y)

% Function doing LOOCV multivariate PCR for considering the maximum
% possible number of PCs.

% -------------------------------------------------------------------------
%
% Inputs:
% X:       Data matrix with rows as individual measurements and colums
%          representing each variable.
% Y:       Corresponding column vector or matrix of response measurements.
%
% -------------------------------------------------------------------------
%
% Outputs:
% BetaLOOCV: The "winning" regression coefficients, including constant
%            term.
% PRESS:     Matrix of PRESS values. Each row represents a response
%            variable, each column a response variable
%
% -------------------------------------------------------------------------


[n, p]     = size(X);         % sample size
m          = size(Y, 2);      % # of responses
A          = min(n, p) - 2;   % max # of components
PRESS      = zeros(n - 2, m); % Init of PRESS
BetaLOOCV  = zeros(p + 1, m);     % Init of BetaLOOCV

% LOOCV
for i = 1:n
    inds = setdiff(1:n, i);                         % nth sample left out
    beta = pcr_int(X(inds, :), Y(inds, :), A); % fast pls for reduced data
    for j = 1:A
        res2 = (bsxfun(@minus, Y(i, :),[1, X(i, :)]*beta(:, :, j))).^2; % squared LOO res
        PRESS(j, :) = PRESS(j, :) + res2;                           % adds to PRESS
    end
end

[~, ncomp] = min(PRESS);  % # of components giving lowest PRESS for each response

Beta       = pcr_int(X, Y, max(ncomp)); % reg coeffs for full data set

for i = 1:m
    BetaLOOCV(:, i) = Beta(:, i, ncomp(i)); % LOOCV reg coeffs
end


function Beta = pcr_int(X,Y,A)
%% Principal component regression with k components.

% Function for computing PCR based on A principal componets

% Written by U. G. Indahl, M. Ansnes has made the following adjustments:
% 1) Removed the option of finding the reg coeffs for just one choice of #
%    of components.
% 2) Expanded the function to work with multivariate response.

% Inits:
[n, p] = size(X);            % sample size
m      = size(Y, 2);         % # of responses
mX     = mean(X);            % means of X columns
X      = X - ones(n, 1)*mX;  % centering of X
Beta   = zeros(p + 1, A, m); % init of Beta

[U, S, V] = svds(X, A);      % extracting first A PCs
s = diag(S);                 % diagonalization of S
for k = 1:m
    b = cumsum(bsxfun(@times, V, (s.\(U'*Y(:, k)))'),2); % reg coeffs for <= A PCs
    Beta(:, :, k) = [mean(Y(:, k)) - mX*b; b];           % reg coeffs with const terms
end
Beta = permute(Beta, [1, 3, 2]); % rearrangement of dimensions for easier use of models
