% Function that does leave one out cross validation on multi-response data
% analyzed with PLSR. Uses the function pls2fast as an internal function,
% to which I have made the following modifications:
%   1. Changed the names of the different dimensions.
%   2. Changed the beta-output so that the first index is a specific
%      regression coefficient, the second index gives the response
%      variable, and the third gives the number of PLS components used.
%--------------------------------------------------------------------------
%
% Inputs:  X - data matrix with rows as measurements and one column for
%              each variable.
%          Y - matrix of measured responses, one column for each response
%              variable.
%
% Outputs: betaLOOCV - matrix of regression coefficients. Each row
%                      represents a set of coefficients for one response 
%                      variable.
%          PRESS     - matrix of PRESS values. Each row represents the
%                      number of PLS components used, and each row
%                      represents a response variable.

function [betaLOOCV, PRESS] = PLS2fastLOOCV(X, Y)

% Inits

[n, p]     = size(X);         % sample size
m          = size(Y, 2);      % # of responses
A          = min(n, p) - 1;   % max # of components
PRESS      = zeros(n - 1, m); % Init of PRESS
betaLOOCV  = zeros(p, m);     % Init of BetaLOOCV

% LOOCV
for i = 1:n
    inds = setdiff(1:n, i);                         % nth sample left out
    beta = pls2fast_int(X(inds, :), Y(inds, :), A); % fast pls for reduced data
    for j = 1:A
        res2 = (bsxfun(@minus, Y(i, :), X(i, :)*beta(:, :, j))).^2; % squared LOO res
        PRESS(j, :) = PRESS(j, :) + res2;                           % adds to PRESS
    end
end

[~, ncomp] = min(PRESS);  % # of components giving lowest PRESS for each response

beta       = pls2fast_int(X, Y, max(ncomp)); % reg coeffs for full data set

for i = 1:m
    betaLOOCV(:, i) = beta(:, i, ncomp(i)); % LOOCV reg coeffs
end

function beta = pls2fast_int(X, Y, A)
% -------------------------------------------------------------------------
% ------------------ Ulf Indahl 11/02-2014 --------------------------------
% -------------------------------------------------------------------------
[n,p] = size(X);
%mX = mean(X); X = bsxfun(@minus, X, mX); % size & mean centering of X.
%mY = mean(Y); Y = bsxfun(@minus, mY,Y);
m = size(Y,2); 
beta = zeros(p,m,A);   % The regression coefficients
q = zeros(m,A);
T  = zeros(n,A);   % Orthonormal scores
W  = zeros(p,A);   % Orthonormal weights
% -------------------- Solution of the PLS-problem ----------------------- 
for a = 1:A
    [w,~,~] = svd(X'*Y,'econ'); w = w(:,1);
     if a > 1
         w = w-W(:,1:a-1)*(W(:,1:a-1)'*w); % Included for numerical stability.
     end
     W(:,a)= w/norm(w);  
     t = X*W(:,a);
     if a > 1
         t = t-T(:,1:a-1)*(T(:,1:a-1)'*t); 
     end
     T(:,a) = t/norm(t);
     q(:,a) = Y'*T(:,a);     % Regression coeffs (Y-loadings) for the orthogonal scores.
     Y = Y - T(:,a)*q(:,a)'; % Y-residual for extraction of next component (Y-deflation)
end
% -------------------------------------------------------------------------
% ---- Postprocessing to find regression coeffs & other key matrices -----
P    = X'*T; % X-loadings.
PtW  = triu(P'*W); % The W-coordinates of (the projected) P.
R    = W/PtW; % The "SIMPLS weights".

for i = 1:m,
    beta(:,i,:) = cumsum(bsxfun(@times,R, q(i,:)),2);
end
