function [Ghat, pclass, a, covi, muG, confmat, pcc] = QDA(X, G)

% Function doing quadratic discriminant analysis on a data matrix X with
% corresponding class membership vector G.
%
% Inputs
% ------
% X: Data matrix consisting of n measurements (rows) and p variable
%    (columns)
% G: Column vector of class memberships for each row in X.
%
% Outputs
% -------
% Ghat:    Predicted class memberships.
% plcass:  Predicted class membership probabilities.
% a:       Coefficients for calculating probabilities.
%          = (2*pi)^(p/2)*sqrt(det(Cov)), where Cov is the class covariance
%          matrix. Needed for prediction.
% covi:    Inverses of covariance matrices for each class. Needed for
%          prediction.
% muG:     Class centres. Needed for prediction.
% confmat: Confusion matrix.
% pcc:     Fraction of correct classifications

% Initializations
[n, p]  = size(X);        % sample size
K       = max(G);         % # of classes
Gd      = dummyvar(G);    % dummy coding of G 
nG      = sum(Gd);        % # of measurements for each class
muG     = (Gd'*Gd)\Gd'*X; % class centres
pclass  = zeros(n, K);    % init of predicted class membership probabilities
a       = zeros(K, 1);    % init of a
d2      = zeros(n, K);    % init of squared Mahalanobis distances matrix 
confmat = zeros(K);       % init of confusion matrix
covi    = cell(K, 1);     % init of covi



for c = 1:K
    Xc      = X(G(:) == c, :);               % data belonging to class c
    Xc      = Xc - ones(nG(c), 1)*muG(c, :); % centered data for class c
    Cov     = 1/(nG(c) - 1)*(Xc'*Xc);        % estimated cov matrix for class c
    a(c)    = (2*pi)^(p/2)*sqrt(det(Cov));   % coefficient needed for probabilities
    covi{c} = pinv(Cov);                     % (pseudo)inverse of cov matrix
end
for i = 1:n
    for c = 1:K
        xbar         = X(i, :) - muG(c, :);       % class centered data point
        d2(i, c)     = xbar*covi{c}*xbar';        % squared Mahalanobis distance
        pclass(i, c) = 1/a(c)*exp(-(d2(i, c))/2); % class membership probability
    end
end
pclass    = bsxfun(@rdivide, pclass, sum(pclass, 2)); % making sums of probs = 1
[~, Ghat] = max(pclass, [], 2);                       % class predictions

for i = 1:n
    j = G(i);                          % actual class membership
    k = Ghat(i);                       % model predicted class membership
    confmat(j, k) = confmat(j, k) + 1; % updates covariance matrix
end

pcc = trace(confmat)/n; % fraction of correct classifications 
