function [Ghat, confmat, pcc, beta] = RDA(X, G, method)

% Function doing discriminant analysis by ordinary least squares
% regression, principal component regression, or partial least squares
% regression. Uses functions PCR2LOOCV and PLS2fastLOOCV.
%
% Inputs
% -------
% X:        Data matrix of n measurements (rows) and p predictor variables
%           (colums).
% G:        Group membership column vector.
% (method): Regression method. 'OLS' (Ordinary Least Squares), 'PCA'
%           (Principal Component Analysis), or 'PLS' (Partial Least
%           Squares). If not specified, function automatically chooses OLS 
%           if n > p, elsewise PLS.
%
% Outputs
% --------
% Ghat:    Model classifications of measurements in X.
% confmat: Confusion matrix.
% pcc:     Fraction of correct classifications. 

Gd      = dummyvar(G); % dummy coding of G
[n, p]  = size(X);     % sample size
K       = max(G);      % # of classes
confmat = zeros(K);    % init of confusion matrix


if nargin == 2
    if n > p
        method = 'OLS'; % sets method to 'OLS' if no method argument and n > p
    else
        method = 'PLS'; % sets method to 'PLS' if no method argument and n =< p 
    end
end

if strcmp(method, 'OLS') == 1     % checks method
    Ghat = X*((X'*X)\(X'*Gd));    % OLS regression predictions
elseif strcmp(method, 'PCA') == 1 % checks method
    beta = PCR2LOOCV(X, Gd);      % PCR coefficients
    Ghat = [ones(n, 1), X]*beta;  % PCR predictions
elseif strcmp(method, 'PLS') == 1 % checks method
    beta = PLS2fastLOOCV(X, Gd);  % PLS regression coefficients
    Ghat = X*beta;                % PLSR predictions
end

[~, Ghat] = max(Ghat, [], 2);     % Model classifications
for i = 1:n
    j = G(i);                          % actual class
    k = Ghat(i);                       % model prediction
    confmat(j, k) = confmat(j, k) + 1; % updates confmat
end
pcc = trace(confmat)/n; % fraction of correct classifications
