function [Beta, Betas, H, GCV, lambda] = Ridge_GCV_SVD(X, Y, lambdas)

% Function finding the least squares solution with Ridge regularization and
% fast leave one out cross validation for a data matrix X and corresponding
% response vector y over an array of lambdas. Uses SVD.
%
% -------------------------------------------------------------------------
%
% Inputs:
% X:       Data matrix with rows as individual measurements and colums
%          representing each variable.
% Y:       Corresponding column vector or matrix of response measurements.
% lambdas: Array of regularization parameter values
%
% -------------------------------------------------------------------------
%
% Outputs:
% beta:   The "winning" regression coefficients, including constant term.
% betas:  Tensor containing matrices with all regression coefficient
%         vectors as rows, one for each lambda value
% H:      Matrix of leverage values, each column represents a lambda value
% PRESS:  Matrix of PRESS values. Each row represents a response
%         variable, each column a response variable
% lambda: The "winning" regularization parameter value for each response
%         variable
%
% -------------------------------------------------------------------------

% Initializations
[n, p] = size(X);           % Sample size
m      = size(Y, 2);        % # of response variables
q      = length(lambdas);   % # of regularization parameter values
H      = zeros(n, q);       % Initialization of H
Betas  = zeros(p, m, q);    % Initialization of Betas
Beta   = zeros(p + 1, m);   % Initialization of Beta
GCV    = zeros(q, m);       % Initialization of PRESS
mX     = mean(X);           % Mean data values, one for each variable
mY     = mean(Y);           % Mean response value
X      = X - ones(n, 1)*mX; % Centered data matrix
Y      = Y - ones(n, 1)*mY; % Centered response values

[U, S, V] = svd(X, 'econ');
S2 = S.^2;

for i = 1:q
    Betas(:, :, i) = V*(S./(S2 + lambdas(i)))*U'*Y;
    Yhat = X*Betas(:, :, i);
    H(:,i) = sum(bsxfun(@times,U,(diag(S./sqrt(S2+lambdas(i))))').^2,2)+1/n;    % LOOCV values
    h = sum(H(:, i));
    resGCV = bsxfun(@rdivide, Y - Yhat, 1 - h/n); % CV fitted values
    GCV(i, :) = sum(resGCV.^2);     % PRESS statistic
end
[~, inds] = min(GCV);
lambda = lambdas(inds);
for i = 1:m
    Beta(:, i) = [mY(i) - mX*Betas(:, i, inds(i)); Betas(:, i, inds(i))];
end
if m == 1
    Betas = permute(Betas, [1, 3, 2]);
end