function [Beta, H, PRESS, lambda] = Ridge_fastLOOCV_fmin(X, Y, lambdas)

% Function finding the least squares solution with Ridge regularization and
% fast leave one out cross validation for a data matrix X and corresponding
% response vector y over an array of lambdas. Uses SVD.
%
% -------------------------------------------------------------------------
%
% Inputs:
% X:       Data matrix with rows as individual measurements and colums
%          representing each variable.
% Y:       Corresponding column vector or matrix of response measurements.
% lambdas: Array consisting of the two boundaries for the lambda interval.
%
% -------------------------------------------------------------------------
%
% Outputs:
% beta:   The "winning" regression coefficients, including constant term.
% betas:  Tensor containing matrices with all regression coefficient
%         vectors as rows, one for each lambda value
% H:      Matrix of leverage values, each column represents a lambda value
% PRESS:  Matrix of PRESS values. Each row represents a response
%         variable, each column a response variable
% lambda: The "winning" regularization parameter value for each response
%         variable
%
% -------------------------------------------------------------------------

% Initializations 
[n, p]  = size(X);           % Sample size
m       = size(Y, 2);        % # of responses
mX      = mean(X);           % Mean data values, one for each variable
mY      = mean(Y);           % Mean response value
X       = X - ones(n, 1)*mX; % Centered data matrix
Y       = Y - ones(n, 1)*mY; % Centered response values
PRESS   = zeros(1, m);       % init of PRESS
lambda  = zeros(1, m);       % init of lambda
Beta    = zeros(p, m);       % init of Beta

[U, S, V] = svd(X, 'econ');
S2 = S.^2;
for k = 1:m
    Betaf = @(lambda) V*(S./(S2 + lambda))*U'*Y(:, k); % reg coeffs
    H = @(lambda) sum(bsxfun(@times,U,(diag(S./sqrt(S2+lambda)))').^2,2)+1/n; % leverage
    res = X*Betaf(lambda) - bsxfun(@times, Y(:, k), H(lambda)); % residuals
    Yhatcv = @(lambda) bsxfun(@rdivide, res, 1 - H(lambda)); % CV fitted values
    PRESSs = @(lambda) sum((Y(:, k) - Yhatcv(lambda)).^2);     % PRESS statistic
    [PRESS(k), lambda(k)] = fminbnd(PRESSs, lambdas(1), lambdas(2)); % PRESS & lambda
    Beta(:, k) = V*(S./(S2 + lambda(k)))*(U'*Y(:, k)); % Winning reg coeffs
end

Beta = [mY - mX*Beta; Beta]; % reg coeffs with constant term
