function [Ghat, GhatCV, d2CV, muG, W, ncc, pcc, confmat] = TLDA_LOOCV(X, G, lambdas, d)

% Function doing d-type Tikhonov regularized LDA of data matrix X and group
% membership vector G over an array of regularization parameter values
% lambdas. Does LOOCV and chooses the model that has the lowest sum of
% Mahalanobis distances from correct class centers from the models that has
% the maximum amount of correct classifications.
%
% This function is a modification of Ulf Indahl's LDA function of March
% 2015 for the MATH310 course at the Norwegian University of Life Sciences.
%
% Inputs
% ------
% X:       Sample matrix of n samples (rows) and p variables (columns).
%          Assumed to contain samples from all groups.
% G:       Vector containing numeric group membership for each sample in X
% lambdas: Array of regularization parameter values. Should be
%          log10-distributed around 1.
% d:       Regulariztaion type. Does Ridge regularization if set to 0,
%          otherwise Tikhonov regularization of nth order derivative if set
%          to n > 0.
%
% Outputs
% -------
% Ghat:    Class predictions for all lambda values
% GhatCV:  CV class predictions for all lambda values
% d2CV:    CV squared Mahalanobis distances
% muG:     Class centers
% W:       Sphering matrix for best choice of regularization parameter
%          lambda
% ncc:     # of correct CV classifications
% pcc:     Fraction of CV correct classifications
% confmat: Confusion matrix for CV predictions


% Initializations
[n, p]  = size(X);         % sample size
K       = max(G);          % # of classes
q       = length(lambdas); % # of regularization parameter values
d2      = zeros(n, K);     % init of squared Mahalanobis distances
d2CV    = zeros(n, K, q);  % init of squared CV Mahalanobis distances
Ghat    = zeros(n, q);     % init of model predictions
GhatCV  = zeros(n, q);     % init of CV model predictions
dist    = zeros(1, q);     % init of winning distances norms
confmat = zeros(K);        % init of confusion matrix
W       = cell(1, q);      % init of sphering matrices


% Regularization of data
if d > 0
    T = diff([speye(p); sparse(d, p)], d); % regularization matrix
    X = X/T;                               % T^{-1} transformed data
end

% Group centering and SVD
Gd        = dummyvar(G);                        % dummy coding of G
muG       = (Gd'*Gd)\Gd'*X;                     % class centres
Xc        = X - muG(G, :);                      % class centered matrix
[~, S, V] = svd(Xc, 'econ');                    % SVD
s         = diag(S);                            % undigaonalisation of S
D         = sqrt(bsxfun(@plus, s.^2, lambdas)); % updated SV's

for c = 1:K
    comp = (speye(p) - V(:, 1:n - K + c - 1)*V(:, 1:n - K + c - 1)')*muG(c, :)'; % orth comp.
    V(:, n - K + c) = comp/norm(comp); % updates V
end

% The LDA: Mahalanobis distances and group membership predictions
for k = 1:q
    W{k} = bsxfun(@rdivide, V, D(:, k)'); % sphering matrix
    for i = 1:n
        for c = 1:K
            d2(i, c) = (X(i, :) - muG(c, :))*W{k}*W{k}'*(X(i, :) - muG(c, :))'; % Mah dist^2
        end
    end
    [~, Ghat(:, k)] = min(d2, [], 2); % model predictions for each lambda
end

% LOOCV

for k = 1:q
    for i = 1:n
        inds           = setdiff(1:n, i);              % leaves out ith index
        Xred           = X(inds, :); Gred = G(inds);   % reduced data
        [Wred, muGred] = LDA_int(Xred, Gred, lambdas); % reduced W and means
        for c = 1:K
            d2CV(i, c, k) = norm((X(i, :) - muGred(c, :))*Wred{k}).^2; % CV Mah dist^2
        end
        d2CV(i, :, k) = d2CV(i, :, k)/sum(d2CV(i, :, k)); % Realtive distance
    end
    [windist, GhatCV(:, k)] = min(d2CV(:, :, k), [], 2); % winning dist's and pred's
    dist(k)                 = norm(windist); % norm of winning dist's
    ncc                     = sum(G == GhatCV(:, k));
end

pcc = ncc/n;
inds = find(ncc == max(ncc));
ranking = pcc(inds) - dist(inds);
[~, idx] = max(ranking);
GhatCV = GhatCV(:, inds(idx));
W = W{inds(idx)};

for i = 1:n
    j = G(i);                          % actual class membership
    k = GhatCV(i);                     % CV predicted class membership
    confmat(j, k) = confmat(j, k) + 1; % updates confmat
end

ncc = trace(confmat); % # of correct classifications
pcc = ncc/n;          % fraction of correct classifications


    

function [Wred, muGred] = LDA_int(X, G, lambdas)
[n, p] = size(X);
K = max(G);
q = length(lambdas);
Gd = dummyvar(G);
Wred = cell(q, 1);
muGred = (Gd'*Gd)\Gd'*X;
Xc = X - muGred(G, :);
[~, S, V] = svd(Xc, 'econ');
s = diag(S);
D = sqrt(bsxfun(@plus, s.^2, lambdas));
for c = 1:K
    comp = (speye(p) - V(:, 1:n - K + c - 1)*V(:, 1:n - K + c - 1)')*muGred(c, :)';
    V(:, n - K + c) = comp/norm(comp);
end
for k = 1:q
    Wred{k} = bsxfun(@rdivide, V, D(:, k)');
end

