function [muG, W, Ghat, GhatCV, dist,...
    ncc_all, pcc_all, lambda, ncc, pcc, confmat] = TLDAfastLOOCV(X, G, lambdas, d)
% Function doing Tikhonov regularized LDA of sample matrix X and group
% membership vector G over an array of regularization parameter values
% lambdas.
%
% This function is an modification of Ulf Indahl's LDA function of March
% 2015 for the MATH310 Course at the Norwegian University of Life Sciences.
%
% Inputs:
% -------------------------------------------------------------------------
% X       - Sample matrix with each row as individual samples, each column
%           represents a variable. X is assumed to contain measurements
%           for all groups.
% G       - Vector containing numeric group memberships for each sample in
%           X.
% lambdas - Array of regularization parameter values. Generally
%           log10-distributed.
% d       - Type of regularization. For non-negative integer values, does
%           the corresponding derivative regularization. Gives the zeroth
%           (Ridge) derivative regularization if = 0.
%
% Outputs:
% -------------------------------------------------------------------------
% Ghat    - Predicted group membership based on Mahalanobis distances from
%           group means.
% GhatCV  - LOOCV predicted group memberships based on Mahalanobis
%           distances from group means.
% dist    - Vector containing the norm of relative Mahalanobis distances
%           for each lambda.
% ncc_all - # of correct LOOCV classifications for each lambda.
% pcc_all - % of correct LOOCV classifications for each lambda.
% lambda  - The "winning" lambda value based on difference in % correct
%           LOOCV classifications and dist value.
% ncc     - # of correct LOOCV classifications for winning lambda
% pcc     - % of correct LOOCV classifications for winning lambda
% confmat - Confusion matrix for the analysis. Rows are actual group
%           memberships, columns are LOOCV predicted.
% -------------------------------------------------------------------------

% Initializations

[n, p]  = size(X);         % Size of data matrix 
K       = max(G);          % # of groups
q       = length(lambdas); % # of lambda values
d2      = zeros(n, K);     % Init of squared Mahalanobis distances
d2CV    = zeros(n, K);     % Init of squared LOOCV Mahalanobis distances
Ghat    = zeros(n, q);     % Init of Ghat
GhatCV  = zeros(n, q);     % Init of GhatCV
dist    = zeros(1, q);     % Init of dist
ncc_all = zeros(1, q);     % Init of ncc_all
confmat = zeros(K);        % Init of confmat
W       = cell(q, 1);      % Init of W

% Regularization of data
if d > 0
    T = diff([speye(p);sparse(d, p)], d); % Regularization matrix
    X = X/T;                              % T^{-1}-transformed data
end

% Group centering of data and SVD
Gd        = dummyvar(G);                % Dummy coding of group memberships
nG        = sum(Gd)';                   % # of values for each group
muG       = (Gd'*Gd)\Gd'*X;             % Group centres
Xc        = X - muG(G, :);              % Data centred wrt group memberships
[~, S, V] = svd(Xc, 'econ');            % Economical SVD of group centred data
s = diag(S);                            % Diagonalization of S
D = sqrt(bsxfun(@plus, s.^2, lambdas)); % Matrix containing lambda-corrected SVs

if K > 2
    D = sparse(D); % Making D sparse if more than 2 groups
end

% The LDA: Mahalanobis distances and group membership predictions
for k = 1:q
    W{k} = V/diag(D(:, k)); % Sphering matrix
    for i = 1:n
        for c = 1:K
            d2(i, c) = norm((X(i, :) - muG(c, :))*W{k})^2; % Squared Mahalanobis distances
        end
    end
    [~, Ghat(:, k)] = min(d2, [], 2); % Predicted group memberships
    % LOOCV squared Mahalanobis distances (Ripley, p. 100 [*]):
    for i = 1:n
        g  = G(i);     % Group membership for jth sample
        nc = nG(g);    % # of samples belonging to group c
        xc = Xc(i, :); % jth sample group centred wrt correct group
        for c = 1:K
            if c == g
                % Ripley's Mahalanobis distance to correct group mean:
                d2CV(i, g) = d2(i, g)*(nc/(nc - 1))^2/...
                (1 - (nc/(nc - 1))*d2(i, g));         
            else
                xk = X(i, :) - muG(c, :); % jth sample group centered wrt incorrect group
                % Ripley's Mahalanobis distance to incorrect group mean:
                d2CV(i, c) = d2(i, c)*(1 + ((xc*W{k})*...
                (xk*W{k})')^2/...
                (((nc - 1)/nc - d2(i, g))*d2(i, c))); 
            end
        end
        d2CV(i, :) = d2CV(i, :)/sum(d2CV(i, :)); % Realtive distance
    end
    [d2CVw, GhatCV(:, k)] = min(d2CV, [], 2); % Winning d2CV distance + predictions
    dist(k) = norm(d2CVw);                    % Norm of relative LOOCV Mahalanobis distance
    ncc_all(k) = sum(G == GhatCV(:, k));      % # of correct LOOCV predictions
end

pcc_all  = ncc_all/n;                  % % of correct LOOCV predictions
ncc      = max(ncc_all);               % Max # of correct LOOCV predictions 
pcc      = ncc/n;                      % Max % of correct LOOCV predictions
inds     = find(ncc_all == ncc);       % Lambda indices with max correct predictions
ranking  = pcc_all(inds) - dist(inds); % Ranking parameter
[~, idx] = max(ranking);               % Index of inds with highest ranking
lambda   = lambdas(inds(idx));         % winning lambda

% Confusion matrix

for k = 1:n
    i = G(k);                      % Actual group membership for kth sample
    j = GhatCV(k, inds(idx));      % Predicted group membership
    confmat(i,j) = confmat(i,j)+1; % Updates confmat accordingly
end

W = W{inds(idx)};

% References:

% [*] Ripley, B. D.: "Pattern Recognition and Neural Networks". Cambridge
%     University Press, 1996.