function [pclass, pclassCV, GhatCV, a, W, muG, confmat, pcc, lambda] = TQDAfastLOOCV(X, G, lambdas, d)
% Attempt of fast LOOCV Tikhonov regularized QDA.

% Inputs
% ------
% X:       Data matrix containing n measurements (rows) of p variables
%          (columns).
% G:       Corresponding class membership vector.
% lambdas: Array of log distributed regularization parameter values.
%          lambdas = 0 gives no regularization.
% d:       Order of differentiation. d = 0 gives Ridge regularization

% Outputs
% -------
% pclass:   model probabilities for all measurements over all lambdas
% pclassCV: LOOCV probabilities for all measurements over all lambdas
% GhatCV:   LOOCV predictions for all measurements over all lambdas
% a:        coefficient for calculating log probabilites, needed for
%           prediction
% W:        winning sphering matrices, needed for prediction
% muG:      class centres, needed for prediction
% confmat:  LOOCV confusion matrix
% pcc:      fractions of correct classifications for all lambdas
% lambda:   winning lambda value

% Initializations
[n, p]      = size(X);         % sample size
q           = length(lambdas); % # of lambda values
K           = max(G);          % # of classes
Gd          = dummyvar(G);     % dummy coding of G 
nG          = sum(Gd);         % # of measurements for each class
muG         = (Gd'*Gd)\Gd'*X;  % class centres
GhatCV      = zeros(n, q);     % init of LOOCV predictions
pclass      = zeros(n, K, q);  % init of probabilities tensor
pclassCV    = zeros(n, K, q);  % init of LOOCV probabilities tensor
logpclass   = zeros(n, K);     % init of predicted class membership probabilities
logpclassCV = zeros(n, K, q);  % init of LOOCV log probabilities
pclassCVw   = zeros(q, 1);     % init of winning probabilities
ncc         = zeros(q, 1);     % init of ncc
a           = zeros(K, q);     % init of a
d2          = zeros(n, K);     % init of squared Mahalanobis distances matrix 
confmat     = zeros(K);        % init of confusion matrix
Ws          = cell(K, q);      % init of Ws
W           = cell(K, 1);      % init of winning W matrices

if d > 0
    T = diff([speye(p);sparse(d, p)], d); % regularization matrix
    X = X/T;                              % T^{-1}-transformed data
end

for c = 1:K
    Xc      = X(G(:) == c, :);               % data belonging to class c
    Xc      = Xc - ones(nG(c), 1)*muG(c, :); % centered data for class c
    [~, S, V] = svd(Xc, 'econ');             % SVD of data in class c
    dimS = length(diag(S));                  % dimension of S
    s = diag(S);                             % vecorization
    S2 = s.^2;                               % squared s
    D2 = bsxfun(@plus, S2, lambdas);         % lambda updated squared sv's
    trD2 = sum(D2); trS2 = sum(S2);          % traces for re-adjusting total variance 
    D2 = bsxfun(@times, D2, trS2./trD2);     % D2 has the same total variance as S2
    D = sqrt(D2);                            % new sv's
    a(c, :) = dimS/2*log(nG(c) - 1) - p/2*log(2*pi) - sum(log(D)); % coeff for pred 
    for k = 1:q
        Ws{c, k} = sqrt(nG(c) - 1)*bsxfun(@rdivide, V, D(:, k)');  % sphering matrix
    end
end
for k = 1:q
    for i = 1:n
        for c = 1:K
            xbar         = X(i, :) - muG(c, :);     % class centered data point
            d2(i, c)     = norm(xbar*Ws{c, k}).^2;  % squared Mahalanobis distance
            logpclass(i, c) = a(c) - (d2(i, c))/2;  % log class membership probability
        end
    end
    pcl = exp(logpclass);                                 % probabilities
    pclass(:, :, k) = bsxfun(@rdivide, pcl, sum(pcl, 2)); % probs now sum to 1
    
    logpclassCV(:, :, k) = logpclass; % CV probs only change for group of omitted point
    for i = 1:n
        g = G(i);                                                        % group of X(i, :)
        const = abs(1 - nG(g)/(nG(g) - 1)^2*d2(i, g));                   % constant for LOOCV upd.
        aCV = a(g, k) - p/2*(log((nG(g) - 1)/(nG(g) - 2)) + log(const)); % coeff for probs
        d2CV = (d2(i, g)*nG(g)^2*(nG(g) - 2)/(nG(g) - 1)^3) / const;     % upd. Mah. dist
        logpclassCV(i, g, k) = aCV - 0.5*d2CV;                           % CV prob
    end
    pcl = exp(logpclassCV(:, :, k));                        % probabilities
    pclassCV(:, :, k) = bsxfun(@rdivide, pcl, sum(pcl, 2)); % probs now sum to 1
    [pclw, GhatCV(:, k)] = max(pclassCV(:, :, k), [], 2);   % winning probs & preds
    pclassCVw(k) = norm(pclw);                              % secondary discr criterium
    ncc(k) = sum(G == GhatCV(:, k));                        % # of correct classifications
end
pcc      = ncc/n;                       % fraction of correct classifications
inds     = find(ncc == max(ncc));       % lambda indices for max corr class 
ranking  = pcc(inds) + pclassCVw(inds); % ranking parameter to be maximized
[~, idx] = max(ranking);                % inds index for best lambda
win      = inds(idx);                   % winning lambda index

for i = 1:n
    j = G(i);                          % actual class membership
    k = GhatCV(i);                     % model predicted class membership
    confmat(j, k) = confmat(j, k) + 1; % updates covariance matrix
end

a = a(:, win);         % winning a's
lambda = lambdas(win); % winning lambda
for c = 1:K
    W{c} = Ws{c, win}; % winning Ws matrices
end