% Function that does Tikhonov regression on data with univariate or
% multivariate response. Uses the internal MATLAB function min() to
% optimize with regard to PRESS.
%
% Inputs:
% -------
% X:       Matrix of data with rows as data points.
% Y:       Matrix or possibly vector of response data
% lambdas: Array of (log-spread) regularization parameter candidates
% type:    Type of regularization; 0 for norm, n > 0 for nth derivative.
%          Default value 0 (Ridge regularization)
% strd:    Omits standardization of data if this value is set to 0.
%          Optional with default set to standardization.
%
% Outputs:
% --------
% Beta:   Matrix of regression coefficients produced by LOOCV. Each column 
%         is a set of coefficients, one for each response variable.
% lambda: The best lambdas for each response produced by LOOCV.
% PRESS:  Matrix of PRESS values for each lambda. Each column represents a 
%         response variable.
% Betas:  Collection of matrices containing all regression coefficients.
%         Each row is a set of regression coefficients for one
%         regularization parameter value, whereas each matrix respresents a
%         response variable.

function [Beta, lambda, PRESS, Betas] = TregsMulti(X, Y, lambdas, d, strd)

% Setting default values
if nargin == 3
    d    = 0;
    strd = 0;
elseif nargin == 4
    strd = 0;
end

% Initializations:
[n, p] = size(X);         % Size of data
m      = size(Y, 2);      % Number of response variables
q      = length(lambdas); % Number of regularization parameter candidates
Betas  = cell(m, 1);      % Init of Betas
PRESS  = zeros(q, m);     % Init of PRESS
Beta   = zeros(p + 1, m); % Init of Beta
inds   = zeros(1, m);     % Init of inds

% Centering of data:
mX = mean(X);           % Mean value of each X column
mY = mean(Y);           % Mean value of each Y column
X  = X - ones(n, 1)*mX; % Centered explanatory data
Y  = Y - ones(n, 1)*mY; % Centered response data

% Regularization type:
if d ~= 0
    T = diff([speye(p);sparse(d, p)], d); % Regularization matrix
    X = X/T;                              % "Regularized" data w/o lambda
end

% Standardization of data:
if strd == 1
    St = std(X);
    X = bsxfun(@rdivide, X, St); % Standardized data
end

% SVD:
[U, S, V] = svd(X, 'econ'); % Economy version of singular value decomp.

% Leverage:
s  = diag(S);                              % "Linearization" of the singular values
s2 = s.^2;                                 % Squaring the singular values
D  = bsxfun(@plus, s2, lambdas);           % Matrix of singular values for each lambda
H  = (U.^2)*bsxfun(@rdivide, s2, D) + 1/n; % Leverage values for each lambda

% Regression coefficients, GCV, and PRESS:

UYS = bsxfun(@times, U'*Y, s); % Matrix product used in the loop ahead

for k = 1:m % Values for response variable i
    Betas{k}        = V*bsxfun(@rdivide, UYS(:, k), D); % Regression coefficients
    res             = bsxfun(@minus, Y(:, k), X*Betas{k}); % Residuals
    PRESS(:, k)     = sum(bsxfun(@rdivide, res, (1-H)).^2)'; % PRESS
    [~, inds(k)]    = min(PRESS(:, k)); % lambdas-indices for lowest PRESS
    beta            = Betas{k}(:, inds(k)); % "Winning" reg coeffs
    if d > 0
        beta = T\beta;  % Transforms beta
    end
    if strd ~= 0
        beta = St'.\beta; % "Unstandardizes" beta
    end
    Beta(:, k)   = [mY(k) - mX*beta; beta]; % Reg coeffs with constant term
end

% Regularization parameter values
lambda = lambdas(inds); % LOOCV regularization parameters for each response
