% Function that does Tikhonov regression on two criteria for data with
% univariate or multivariate response. Uses the internal MATLAB function
% min() to optimize with regard to PRESS. Keeps the best mu parameter to
% find the best lambda parameter under the assumption that the two
% parameters are independent.
%
% Inputs:
% -------
% X:       Matrix of data with rows as data points.
% Y:       Matrix or possibly vector of response data
% mus:     Array of (log-spread) first regularization parameter candidates
% lambdas: Array of (log-spread) second regularization parameter candidates
% d1:      First regularization type
% d2:      Second regularization type
%
% Outputs:
% --------
% Beta:   Matrix of regression coefficients produced by LOOCV. Each column 
%         is a set of coefficients, one for each response variable.
% mu:     The best mus for each resopnse produced by LOOCV.
% lambda: The best lambdas for each response produced by LOOCV.
% PRESS:  Matrix of PRESS values for each lambda. Each column represents a 
%         response variable.

function [Beta, mu, lambda, PRESS] = TregsMulti2C(X, Y, lambdas, mus, d1, d2)

% Initializations
p      = size(X, 2);      % # of predictor variables 
m      = size(Y, 2);      % # of responses
q      = length(lambdas); % # of lambda values
mX     = mean(X);         % mean values of each X column
mY     = mean(Y);         % mean values of each Y column
Beta   = zeros(p + 1, m); % init of Beta
lambda = zeros(1, m);     % init of lambda
PRESS  = zeros(q, m);     % init of PRESS
Z      = cell(m, 1);      % init of Z

% First d1-regression
[~, mu] = TregsMulti(X, Y, mus, d1); % returns best mu regul. param for first regression

% Second d2-regression
T = diff([speye(p); sparse(d2, p)], d2); % regularization matrix
Y = [Y; zeros(p, m)];                    % augmented Y matrix
for k = 1:m
    Z{k} = [X;sqrt(mu(k))*T];            % augmented data matrix
    % Tikh. regression for kth response:
    [Beta(1:p +1, k), lambda(k), PRESS(:, k)] = TregsMulti(Z{k}, Y(:, k), lambdas, d2); % Treg for kth resp.
    Beta(1, k) = mY(k) - mX*Beta(2:end, k); % reg coeffs with constant term
end
