% Function that does Tikhonov regression on two criteria for data with
% univariate or multivariate response. Uses the internal MATLAB function
% min() to optimize with regard to PRESS.
%
% Inputs:
% -------
% X:       Matrix of data with rows as data points.
% Y:       Matrix or possibly vector of response data
% mus:     Array of (log-spread) first regularization parameter candidates
% lambdas: Array of (log-spread) second regularization parameter candidates
% d1:      First regularization type
% d2:      Second regularization type
%
% Outputs:
% --------
% Beta:   Matrix of regression coefficients produced by LOOCV. Each column 
%         is a set of coefficients, one for each response variable.
% mu:     The best mus for each resopnse produced by LOOCV.
% lambda: The best lambdas for each response produced by LOOCV.
% PRESS:  Matrix of PRESS values for each lambda. Each column represents a 
%         response variable.

function [Beta, mu, lambda, PRESS] = TregsMulti2Cr(X, Y, lambdas, mus, d1, d2)
% Initializations
[n, p] = size(X);           % sample size
m      = size(Y, 2);        % # of response variables
q      = length(lambdas);   % # of lambda values
r      = length(mus);       % # of mu values
mX     = mean(X);           % means of X columns
mY     = mean(Y);           % means of Y columns
X      = X - ones(n, 1)*mX; % centers X
Y      = Y - ones(n, 1)*mY; % centers Y
mu     = zeros(m, 1);       % init of mu
lambda = zeros(m, 1);       % init of lambda
Beta   = zeros(p, m);       % init of Beta
PRESS  = cell(m, 1);        % init of PRESS
Betas  = cell(m, 1);        % init of Betas

for i = 1:m
    PRESS{m} = zeros(q, r);    % fills PRESS cells with zero matrices
    Betas{m} = zeros(p, q, r); % fills Betas cells with zero matrices
end

% Augmentation of data

T2 = diff([speye(p); sparse(d2, p)], d2); % regul. type 2 transf. matrix


Y = [Y; zeros(p, m)]; % Y augmented
Z = [X; full(T2)];    % X augmented

% Transform of Z if regularization type 1 is > 0
if d1 > 0
    T1 = diff([speye(p); sparse(d1, p)], d1); % regul. type 1 transf. matrix
    Z  = Z/T1;                                % transformed Z
end

Xi = Z; % use Xi in loop to update data matrix more easily
for i = 1:r
    Xi(n + 1:end, :) = sqrt(mus(i))*Z(n + 1:end, :);         % mu update
    [U, S, V]        = svd(Xi, 'econ');                      % svd
    s                = diag(S);                              % row of sv's
    S2               = s.^2;                                 % row of squared sv's
    D                = bsxfun(@plus, S2, lambdas);           % regularized sv's
    H                = (U.^2)*bsxfun(@rdivide, S2, D) + 1/n; % leverage values
    UYs              = bsxfun(@times, U'*Y, s);              % matrix prod in loop
    for h = 1:m
        Betas{h}(:, :, i) = V*bsxfun(@rdivide, UYs(:, h), D);   % reg coeffs
        res = bsxfun(@minus, Y(:, h), Xi*Betas{h}(:, :, i));    % residuals
        PRESS{h}(:, i) = sum(bsxfun(@rdivide, res, (1-H)).^2)'; % PRESS values
    end
end
for i = 1:m
    [~, idx]     = min(PRESS{i}(:));        % vector index of minimal PRESS value
    [lidx, midx] = ind2sub([q, r], idx);    % matrix indices of minimal PRESS value
    mu(i)        = mus(midx);               % mu value
    lambda(i)    = lambdas(lidx);           % lambda value
    Beta(:, i)   = Betas{i}(:, lidx, midx); % reg coeffs for lowest PRESS
    if d1 > 0
        Beta(:, i) = T1\Beta(:, i);         % retransformed reg coeffs
    end
end
Beta = [mY - mX*Beta; Beta]; % reg coeffs with constant terms
