function [Ghat, V, pcc, confmat, Ghat_test, pcc_test, confmat_test] = class_numbers_svd(Xtrain, Gtrain, vs, Xtest, Gtest)

% Function classifying handwritten digits based on SVD bases. Uses the
% same number of bases for all digits and chooses this number based on
% correct classifications of training data. Predicts a test set if one is
% given.
%
% Inputs
% ------
% Xtrain:  Matrix of n images (rows of stacked transposed pixel columns)
%          each consisting of p pixels/variables (colums)
% Gtrain:  Corresponding column vector of class memberships
% vs:      Maximum number of basis images considered
% (Xtest): Test data
% (Gtest): Test data class memberships
%
%
% Outputs
% -------
% Ghat:
% V:
% pcc:         Fraction of correct classifications for training data
% (Ghat_test): Test data prediction (if test data provided) 
% (pcc_test):  Fraction of correct classifications of test data (if given)

% Initializations
[n, p]    = size(Xtrain);   % sample size
K         = max(Gtrain);    % # of classes
V         = cell(K, 1);     % init of collection of basis vector matrices
ncc       = zeros(vs, 1);    % init of correct classifications vector
res_sums2 = zeros(n, K, vs); % init of squared residuals tensor
Ghat      = zeros(n, vs);    % init of training data predictions matrix
confmat   = zeros(K);

for c = 1:K
    A = Xtrain(Gtrain == c, :);    % data points for class c
    [~, ~, V{c}] = svd(A, 'econ'); % basis image vector matrix
    for j = 1:vs
        comp = eye(p) - V{c}(:, 1:j)*V{c}(:, 1:j)'; % orth. comp of basis
        res = Xtrain*comp';                         % residuals
        res_sums2(:, c, j) = sum(res.^2, 2);        % sum of res.^2 over n
    end
end

for j = 1:vs
    [~, Ghat(:, j)] = min(res_sums2(:, :, j), [], 2); % training data pred's
    ncc(j)          = sum(Gtrain == Ghat(:, j));       % # of corr. classifications
end

[~, bvs] = max(ncc); % max # of correct classifications & # of basis vectors needed
pcc        = ncc/n;    % fractions of correct classifications

Ghat = Ghat(:, bvs);


for c = 1:K
    V{c} = V{c}(:, 1:bvs); % V matrices updated to contain bvs column vectors
end

for i = 1:n
    j = Gtrain(i);
    k = Ghat(i);
    confmat(j, k) = confmat(j, k) + 1;
end


if nargin > 3 % i.e. if test set is given
    confmat_test = zeros(K);
    n         = size(Xtest, 1); % # of samples in test set
    res_sums2 = zeros(n, K);    % new init of res_sums2
    for c = 1:K
        comp            = eye(p) - V{c}*V{c}'; % orthogonal complement of V{c}
        res             = Xtest*comp';         % residuals
        res_sums2(:, c) = sum(res.^2, 2);      % sum of squared residuals
    end
    [~, Ghat_test] = min(res_sums2, [], 2);     % test set predictions
    pcc_test       = sum(Ghat_test == Gtest)/n; % fraction of corr. class. for test data
    for i = 1:n
        j = Gtest(i);
        k = Ghat_test(i);
        confmat_test(j, k) = confmat_test(j, k) + 1;
    end
end
