function [Beta, pcc, confmat, Ghat_test, pcc_test] = class_rand_cont_Ridge(Xtrain, Gtrain, Rdim, lambdas, Xtest, Gtest);

% Function doing image classifications of a data matrix X consisting of n
% vectorized images (trasposed stacked pixel columns) as rows and a
% corresponding class membership column vector G. Also predicts a test set
% if one is given. Adds Rdim randomly generated features and does a full
% LOOCV multivariate regression classification. Does a Ridge regularization
% if a lambda array is given. lambda = 0 gives no regularization. Predicts
% test set if given. Calls the functions TregsMulti and (or)
% OLS_fastLOOCV_SVD.
%
% Inputs
% ------
% Xtrain:  Data matrix consisting of n vectorized images (transposed
%          stacked pixel columns) of p pixels/variables
% Gtrain:  Corresponding class membership column vector
% Rdim:    # of randomly generatet features added
% lambdas: Array of perferrably log-distributed regularization parameter
%          values. 
% (Xtest): Optional test set data matrix
% (Gtest): Optional test set class membership vector
%
% Outputs
% -------
% GhatCV:      LOOCV predictions of training data
% pcc:         Fraction of correct classifications for LOOCV model (and
%              test data)
% confmat:     Confusion matrices for LOOCV model of training data and test
%              data predictions if applicable
% Ghat_testCV: Predictions of test data

% Initializations
p    = size(Xtrain, 2);     % sample size
K         = max(Gtrain);      % # of classes



R          = randi([0, 1], p, Rdim); % random matrix of 0 and 1 entries
R(R == 0)  = -1;                     % setting zero entries in R to -1
XR         = Xtrain*R;               % rows of 1000 randomly generated features
XR(XR < 0) = 0;                      % setting negative feature vaules to 0
Xaug       = [Xtrain, XR];           % augmented data matrix


[Beta, ~, pcc, confmat] = TregsMultiClass(Xaug, Gtrain, lambdas, 0, 0); % Ridge reg coeffs

if nargin == 6
    n                = size(Xtest, 1);               % # of images in test set
    confmat_test     = zeros(K);                     % init of confmat_test
    XR               = Xtest*R;                      % rows of randomly gen. feat's
    XR(XR < 0)       = 0;                            % setting negative entries = 0
    GhatCVs          = [ones(n, 1), Xtest, XR]*Beta; % test set reg predictions
    [~, Ghat_test] = max(GhatCVs, [], 2);          % test set classifications
    for i = 1:n
        j                  = Gtest(i);               % actual class membership
        k                  = Ghat_test(i);         % predicted class membership
        confmat_test(j, k) = confmat_test(j, k) + 1; % updates confmat_test
    end
    pcc_test     = trace(confmat_test)/n; % fractions of correct classifications
    confmat = {confmat, confmat_test};      % confusion matrices
end








