function [Ghat, pclass, a, Ws, muG, confmat, pcc] = logQDAsvd(X, G)

% Function doing quadratic discriminant analysis on a data matrix X with
% corresponding class membership vector G.
%
% Inputs
% ------
% X: Data matrix consisting of n measurements (rows) and p variable
%    (columns)
% G: Column vector of class memberships for each row in X.
%
% Outputs
% -------
% Ghat:    Predicted class memberships.
% plcass:  Predicted class membership probabilities.
% a:       Coefficients for calculating probabilities.
%          = ((nG(c) - 1)/(2*pi))^(p/2)*prod(diag(S))^(-1), where S is the 
%          contains the singular values for class c. Needed for prediction.
% Ws:      Sphering matrices for each class. Needed for prediction.
% muG:     Class centres. Needed for prediction.
% confmat: Confusion matrix.
% pcc:     Fraction of correct classifications

% Initializations
[n, p]    = size(X);        % sample size
K         = max(G);         % # of classes
Gd        = dummyvar(G);    % dummy coding of G 
nG        = sum(Gd);        % # of measurements for each class
muG       = (Gd'*Gd)\Gd'*X; % class centres
logpclass = zeros(n, K);    % init of predicted class membership probabilities
a         = zeros(K, 1);    % init of a
d2        = zeros(n, K);    % init of squared Mahalanobis distances matrix 
confmat   = zeros(K);       % init of confusion matrix
Ws        = cell(K, 1);     % init of Ws

for c = 1:K
    Xc      = X(G(:) == c, :);               % data belonging to class c
    Xc      = Xc - ones(nG(c), 1)*muG(c, :); % centered data for class c
    [~, S, V] = svd(Xc, 'econ');             % SVD of data in class c
    Ws{c} = sqrt(nG(c) - 1)*V/S;             % sphering matrix
    a(c) = p/2*log((nG(c) - 1)/(2*pi)) - sum(log(diag(S))); % coeff for pred 
end

for i = 1:n
    for c = 1:K
        xbar         = X(i, :) - muG(c, :);     % class centered data point
        d2(i, c)     = norm(xbar*Ws{c}).^2;     % squared Mahalanobis distance
        logpclass(i, c) = a(c) - (d2(i, c))/2;  % log class membership probability
    end
end

pclass    = exp(logpclass);                           % getting positive values
pclass    = bsxfun(@rdivide, pclass, sum(pclass, 2)); % making sums of probs = 1
[~, Ghat] = max(pclass, [], 2);                       % class predictions

for i = 1:n
    j = G(i);                          % actual class membership
    k = Ghat(i);                       % model predicted class membership
    confmat(j, k) = confmat(j, k) + 1; % updates covariance matrix
end

pcc = trace(confmat)/n; % fraction of correct classifications 
